#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/ether.h>
#include <linux/wireless.h>

#define exit(a) do { fflush(stdout); fflush(stderr); _exit(a); } while(0)

#define BSSID_SIZE	6
#define SSID_MAX	32

static int apvalid(unsigned char *ap)
{
	int i;

	for(i=0;i<BSSID_SIZE;i++)if(ap[i])break;
	if(i==BSSID_SIZE)return 0;
	for(i=0;i<BSSID_SIZE;i++)if(ap[i]!=0x44)break;
	if(i==BSSID_SIZE)return 0;
	for(i=0;i<BSSID_SIZE;i++)if(ap[i]!=0xff)break;
	if(i==BSSID_SIZE)return 0;
	return 1;
}

static int getessid(char *ifc)
{
	int s;
	struct iwreq wrq;
	char ssid[SSID_MAX+1];

	if((s=socket(AF_INET6,SOCK_DGRAM,0))==-1)
	{
		perror("socket");
		return -1;
	}

	memset(ssid,0,sizeof(ssid));
	memset(&wrq,0,sizeof(wrq));
	wrq.u.data.pointer=ssid;
	wrq.u.data.length=sizeof(ssid);
	strncpy(wrq.ifr_name,ifc,IFNAMSIZ);

	if(ioctl(s,SIOCGIWESSID,&wrq))
	{
		perror("ioctl SIOCGIWESSID");
		close(s);
		return -1;
	}

	close(s);

	printf("%s\n",ssid);

	return 0;
}

static int getbssid(char *ifc)
{
	int s;
	int valid;
	struct iwreq wrq;
	struct ether_addr *eth;
	unsigned char ap[BSSID_SIZE];
	char bfr[1024];

	if((s=socket(AF_INET6,SOCK_DGRAM,0))==-1)
	{
		perror("socket");
		return -1;
	}

	memset(&wrq,0,sizeof(wrq));
	wrq.u.data.pointer=bfr;
	wrq.u.data.length=sizeof(bfr);
	strncpy(wrq.ifr_name,ifc,IFNAMSIZ);

	if(ioctl(s,SIOCGIWAP,&wrq))
	{
		perror("ioctl SIOCGIWAP");
		close(s);
		return -1;
	}

	eth=(struct ether_addr *)&wrq.u.ap_addr.sa_data;
	memcpy(ap,eth->ether_addr_octet,BSSID_SIZE);
	valid=apvalid(ap);

	close(s);

	if(!valid)printf("00:00:00:00:00:00\n");
	else
	{
		for(s=0;s<BSSID_SIZE;s++)printf("%s%02x",s?":":" ",ap[s]);
		printf("\n");
	}

	return 0;
}

static void usage(void)
{
	fprintf(stderr,"Usage: netutil fdnumber\n"
			"       netutil -u fdnumber\n"
			"       netutil -e device\n"
			"       netutil -b device\n"
			"   fdnumber  exclusively lock specified given fd\n"
			"-u fdnumber  unlock given fd\n"
			"-e device    print ESSID for specified device\n"
			"-b device    print BSSID for specified device\n");
}

int main(int argc,char *argv[])
{
	int c;
	char *p;

	while((c=getopt(argc,argv,"e:b:u:"))!=-1)switch(c)
	{
	case 'e':
		c=getessid(optarg);
		exit(c?1:0);

	case 'b':
		c=getbssid(optarg);
		exit(c?1:0);

	case 'u':
		errno=0;
		c=(int)strtol(optarg,&p,10);
		if(errno||p==optarg||(p&&*p))
		{
			usage();
			exit(1);
		}

		while(flock(c,LOCK_UN))if(errno!=EINTR)
		{
			perror("flock");
			exit(1);
		}

		exit(0);

	default:fprintf(stderr,"\n");
		usage();
		exit(1);
	}

	if(optind+1==argc)
	{
		errno=0;
		c=(int)strtol(argv[optind],&p,10);
		if(errno||p==argv[optind]||(p&&*p))
		{
			usage();
			exit(1);
		}

		while(flock(c,LOCK_EX))if(errno!=EINTR)
		{
			perror("flock");
			exit(1);
		}
	}

	exit(0);
}
